/*************************************************************************
 * LICENSE AGREEMENT FOR SAMPLE CODE OR ADDITIONAL FUNCTIONALITY
 * SAS INSTITUTE INC. IS PROVIDING YOU WITH THE COMPUTER SOFTWARE CODE INCLUDED WITH
 * THIS AGREEMENT ("CODE") ON AN "AS IS" BASIS, AND AUTHORIZES YOU TO USE THE CODE SUBJECT
 * TO THE TERMS HEREOF. BY USING THE CODE, YOU AGREE TO THESE TERMS. YOUR USE OF THE CODE
 * IS AT YOUR OWN RISK. SAS INSTITUTE INC. MAKES NO REPRESENTATION OR WARRANTY, EXPRESS OR
 * IMPLIED, INCLUDING, BUT NOT LIMITED TO, WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, NONINFRINGEMENT AND TITLE, WITH RESPECT TO THE CODE.
 *
 * The Code is intended to be used solely as an example for use with part of a product ("Software")
 * you currently have licensed from SAS Institute Inc. or one of its subsidiaries or authorized
 * agents ("SAS"). The Code is designed to add functionality to the Software, but has not necessarily
 * been tested. Accordingly, SAS makes no representation or warranty that the Code will operate
 * error-free. SAS is under no obligation to maintain or support the Code.
 *
 * Neither SAS nor its licensors shall be liable to you or any third party for any general, special,
 * direct, indirect, consequential, incidental or other damages whatsoever arising out of or related
 * to your use or inability to use the Code, even if SAS has been advised of the possibility of such
 * damages.
 *
 * Except as otherwise provided above, the Code is governed by the same agreement that governs the
 * Software. If you do not have an existing agreement with SAS governing the Software, you may not
 * use the Code.
 *
 * SAS and all other SAS Institute Inc. product or service names are registered trademarks or
 * trademarks of SAS Institute Inc. in the USA and other countries. (r) indicates USA registration.
 * Other brand and product names are registered trademarks or trademarks of their respective companies.
 *
 * Copyright 2007, SAS Institute Inc., Cary, NC
 *************************************************************************/
import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.MalformedURLException;
import java.net.URL;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.Set;

import com.sas.drugdev.remote.ObjectModifiedException;
import com.sas.drugdev.remote.PropertyValidationException;
import com.sas.drugdev.remote.RemoteException;
import com.sas.drugdev.remote.ServiceManager;
import com.sas.drugdev.remote.admin.GroupBean;
import com.sas.drugdev.remote.admin.GroupService;
import com.sas.drugdev.remote.admin.InsufficientPrivilegesException;
import com.sas.drugdev.remote.admin.InvalidGroupBeanException;
import com.sas.drugdev.remote.admin.InvalidGroupException;
import com.sas.drugdev.remote.admin.InvalidPasswordException;
import com.sas.drugdev.remote.admin.InvalidUserException;
import com.sas.drugdev.remote.admin.Policy;
import com.sas.drugdev.remote.admin.RemoteGroup;
import com.sas.drugdev.remote.admin.User;
import com.sas.drugdev.remote.admin.UserBean;
import com.sas.drugdev.remote.admin.RemoteUser;
import com.sas.drugdev.remote.admin.UserService;
import com.sas.drugdev.remote.repository.AclRecurseBehavior;
import com.sas.drugdev.remote.repository.ContainerBean;
import com.sas.drugdev.remote.repository.InvalidNodeBeanException;
import com.sas.drugdev.remote.repository.InvalidNodeException;
import com.sas.drugdev.remote.repository.InvalidTypeException;
import com.sas.drugdev.remote.repository.InvalidVersionException;
import com.sas.drugdev.remote.repository.Node;
import com.sas.drugdev.remote.repository.NodeBean;
import com.sas.drugdev.remote.repository.NodeId;
import com.sas.drugdev.remote.repository.NodeModifiedException;
import com.sas.drugdev.remote.repository.PermissionException;
import com.sas.drugdev.remote.repository.RemoteContainer;
import com.sas.drugdev.remote.repository.RemoteFile;
import com.sas.drugdev.remote.repository.RemoteNode;
import com.sas.drugdev.remote.repository.RepositoryService;
import com.sas.drugdev.remote.repository.ValidationException;
import com.sas.drugdev.remote.repository.security.AccessControlEntry;
import com.sas.drugdev.remote.repository.security.AccessControlList;
import com.sas.drugdev.remote.repository.security.Permission;
import com.sas.drugdev.remote.session.AuthenticationException;
import com.sas.drugdev.remote.session.Credentials;
import com.sas.drugdev.remote.session.InvalidSessionException;
import com.sas.drugdev.remote.session.PasswordExpiredException;
import com.sas.drugdev.remote.session.Session;
import com.sas.drugdev.remote.session.SessionFactory;
import com.sas.drugdev.remote.session.UnsupportedCredentialsException;
import com.sas.drugdev.remote.session.UserInactiveException;
import com.sas.drugdev.remote.session.UserRetiredException;
import com.sas.drugdev.remote.session.UsernamePasswordCredentials;
import com.sas.drugdev.remote.type.Type;
import com.sas.drugdev.remote.type.TypeBean;
import com.sas.drugdev.remote.type.TypeService;


public class SASDrugDevRemoteAPIAdminSample
{
	private static final int COMMAND_GET_USER=1;
    private static final int COMMAND_CREATE_USER=2;
    private static final int COMMAND_USER_GET_POLICIES=3;
    private static final int COMMAND_USER_UPDATE_POLICIES=4;
    private static final int COMMAND_USER_GET_PROPS=5;
    private static final int COMMAND_USER_SET_PROPS=6;
    private static final int COMMAND_GET_GROUP=7;
    private static final int COMMAND_CREATE_GROUP=8;
    private static final int COMMAND_GET_MEMBERS_OF_GROUP=9;
    private static final int COMMAND_UPDATE_MEMBERS_OF_GROUP=10;
    private static final int COMMAND_CHANGE_CURRENT_PASSWORD=11;
    private static final int COMMAND_GET_PASSWORD_EXPIRATION_DATE=12;
    private static final int COMMAND_TEST_SESSION=13;
    private static final int COMMAND_EXIT=14;

    private Session session;
    private ServiceManager serviceMgr;
    private UserService userService;
    private GroupService groupService;

    public SASDrugDevRemoteAPIAdminSample(URL serverURL, String userName, String password)
    {
        init(serverURL, userName, password);
    }

    // Initialize the SASDrugDev Remote API client
    private void init(URL serverURL, String userName, String password)
    {
        // Create a new session
        session = createSASDrugDevSession(serverURL, userName, password);

        // Check for a valid session and initialize service manager
        if ( session.isValid())
        {
            serviceMgr = session.getServiceManager();
            userService = serviceMgr.getUserService();
            groupService = serviceMgr.getGroupService();
        }
    }

    // Checks if the current connection to to the SAS Drug Dev server is valid.
    public boolean isConnected() {
        return session.isValid();
    }

    // Create session/open connection to SDD server
    public Session createSASDrugDevSession(URL serverURL, String userName, String password)
    {
        // Create a session using the given user credentials
        Session session = null;
        try {

            Credentials credentials = new UsernamePasswordCredentials(userName, password.toCharArray());
            session = SessionFactory.newSession(serverURL, credentials);

        } catch (AuthenticationException e) {
            // Handle exception
            System.err.println(e.getMessage());
        } catch (PasswordExpiredException e) {
            // Handle exception
            System.err.println(e.getMessage());
        } catch (UserInactiveException e) {
            // Handle exception
            System.err.println(e.getMessage());
        } catch (UserRetiredException e) {
            // Handle exception
            System.err.println(e.getMessage());
        } catch (RemoteException e) {
            // Handle exception
            System.err.println(e.getMessage());
        } catch (UnsupportedCredentialsException e) {
            // Handle exception
            System.err.println(e.getMessage());
        }
        return session;
    }

    // Close connection to SDD
    public void closeSASDrugDevSession()
    {
        // check if session is valid before closing server session
        if ( session.isValid())
        {
            // invalidate session and free up server side resources held by this session
            session.invalidate();
        }
    }


 // Sample code to get user properties
    public void getUserProperties(String userId)
    {
        try {
            RemoteUser user = userService.get(userId);

            if ( user != null ) {
                System.out.println("\nGetting properties for user "+ user.getUserId());
                Map properties = user.getProperties();

                // lookup properties by name

                // Iterate through properties
                for (Iterator iter = properties.keySet().iterator(); iter.hasNext();) {

                    String propertyName = (String) iter.next();
                    String propertyValue = (String)properties.get(propertyName);

                    // do something with property, for ex: print them out
                    System.out.println(propertyName + " = " + propertyValue);
                }
            } else {
                System.out.println("User doesn't exist: " + user.getUserId());
            }

        } catch (RemoteException e) {
            // Handle exception
            System.err.println(e.getMessage());
        } catch (InvalidSessionException e) {
            // Handle exception
            System.err.println(e.getMessage());
        } catch (InsufficientPrivilegesException e) {
            // Handle exception
            System.err.println(e.getMessage());
        }
    }

    // Example to show how user properties can be modified individually
    public void setUserProperties(String userId, String propertyName, String propertyValue)
    {
        try {
        	RemoteUser user = userService.get(userId);

            if ( user != null)
            {
            	UserBean userBean = new UserBean(user);
                // set single property
            	userBean.setProperty(propertyName, propertyValue);

                user = userService.update(userBean);
                System.out.println("Modified " + propertyName + " to " + propertyValue.toString());
            } else {
                System.out.println("User doesn't exist: " + user.getUserId());
            }


        } catch (RemoteException e) {
            // Handle exception
            System.err.println(e.getMessage());
        } catch (InvalidUserException e) {
            // Handle exception
            System.err.println(e.getMessage());
        } catch (InsufficientPrivilegesException e) {
            // Handle exception
            System.err.println(e.getMessage());
        } catch (InvalidSessionException e) {
            // Handle exception
            System.err.println(e.getMessage());
        } catch (ObjectModifiedException e) {
            // Handle exception
            System.err.println(e.getMessage());
        } catch (PropertyValidationException e) {
            // Handle exception
            System.err.println(e.getMessage());
        }

    }

    // Create a new user in SDD
    public void createUser(String userId, String password, String firstName, String lastName, String email)
    {
        try {

            // Construct a UserBean with the required properties

        	UserBean userBean = new UserBean(userId);
            userBean.setPassword(password.toCharArray());
            userBean.setFirstName(firstName);
            userBean.setLastName(lastName);
            userBean.setEmailAddress(email);

            // Create this user on the server
            RemoteUser remoteUser = userService.create(userBean);

            // Create successful, use remoteFile to perform other actions on this file.
            System.out.println("\n Created user = " + remoteUser.getUserId());
            System.out.println("\n First name = " + remoteUser.getFirstName());
            System.out.println("\n Last name = " + remoteUser.getLastName());
            System.out.println("\n Email = " + remoteUser.getEmailAddress());
        } catch (Exception e) {
            // Handle exception
            System.err.println(e.getMessage());
        }

    }

//  Get a user from SDD.
    public void getUser(String userId)
    {
        try {
			RemoteUser remoteUser = userService.get(userId);
		   	if (remoteUser != null){
		   		System.out.println();
				System.out.println("Got User: " + remoteUser.getUserId());
				System.out.println("\n First name = " + remoteUser.getFirstName());
				System.out.println("\n Last name = " + remoteUser.getLastName());
            	System.out.println("\n Email = " + remoteUser.getEmailAddress());
		   	} else {
			  	System.out.println("User doesn't exist.");
			}

        } catch (InsufficientPrivilegesException e) {
            // Handle exception
            System.err.println(e.getMessage());
        } catch (RemoteException e) {
            // Handle exception
            System.err.println(e.getMessage());
        } catch (InvalidSessionException e) {
            // Handle exception
            System.err.println(e.getMessage());
        }
    }

    // Sample code to get user policies
    public void getUserPolicies(String userId)
    {
        try {
            RemoteUser user = userService.get(userId);

            if ( user != null ) {
                System.out.println("\nGetting policies for user "+ user.getUserId());
                List policies = user.getPolicies();

                // Iterate through policies
                if (policies.isEmpty()) {
                	System.out.println("User has no policies.");
				} else{
					for (Iterator iter = policies.iterator(); iter.hasNext();) {

						Policy policy = (Policy) iter.next();

						// do something with policy, for ex: print them out
						System.out.println(policy.toString());
					}
				}
            } else {
                System.out.println("User doesn't exist: " + user.getUserId());
            }

        } catch (RemoteException e) {
            // Handle exception
            System.err.println(e.getMessage());
        } catch (InvalidSessionException e) {
            // Handle exception
            System.err.println(e.getMessage());
        } catch (InsufficientPrivilegesException e) {
            // Handle exception
            System.err.println(e.getMessage());
        }
    }

//  Sample code to get user policies
    public void updateUserPolicies(String userId)
    {
        try {
            RemoteUser user = userService.get(userId);

            if ( user != null ) {
                System.out.println("\nGetting policies for user "+ user.getUserId());

                // Set user policies
                UserBean userBean = new UserBean(user);
                userBean.addPolicy(Policy.USER_POLICY_DATA_EXPLORER);
                userBean.addPolicy(Policy.USER_POLICY_JOB_EDITOR);
                user = userService.update(userBean);
                List updatedPolicies = user.getPolicies();

                // Iterate through updated policies

                for (Iterator iter = updatedPolicies.iterator(); iter.hasNext();) {

                    Policy policy = (Policy) iter.next();

                    // do something with policy, for ex: print them out
                    System.out.println(policy.toString());
                }

            } else {
                System.out.println("User doesn't exist: " + user.getUserId());
            }

        } catch (RemoteException e) {
            // Handle exception
            System.err.println(e.getMessage());
        } catch (InvalidSessionException e) {
            // Handle exception
            System.err.println(e.getMessage());
        } catch (InsufficientPrivilegesException e) {
            // Handle exception
            System.err.println(e.getMessage());
        } catch (ObjectModifiedException e) {
            // Handle exception
            System.err.println(e.getMessage());
        } catch (PropertyValidationException e) {
            // Handle exception
            System.err.println(e.getMessage());
        } catch (InvalidUserException e) {
            // Handle exception
            System.err.println(e.getMessage());
        }
    }

    //get a group with the given name from SDD
    public void getGroupFromSDD(String groupName){
    	try{
    		RemoteGroup group = groupService.get(new GroupBean(groupName));

    		System.out.println("Group Name: " + group.getName());
    		System.out.println("Description: " + group.getDescription());
    	}
    	catch(InvalidSessionException e){
            // Handle exception
            System.err.println(e.getMessage());
    	}
    	catch(InsufficientPrivilegesException e){
            // Handle exception
            System.err.println(e.getMessage());
    	}
    	catch(InvalidGroupException e){
            // Handle exception
            System.err.println(e.getMessage());
    	}
    	catch(RemoteException e){
            // Handle exception
            System.err.println(e.getMessage());
    	}
    	catch(InvalidGroupBeanException e){
            // Handle exception
            System.err.println(e.getMessage());
    	}
    }

    //create a new group in SDD
    public RemoteGroup createGroup(String name, String description){
    	try{
    		GroupBean groupBean = new GroupBean(name);
    		groupBean.setDescription(description);

    		RemoteGroup ret = groupService.create(groupBean);

    		System.out.println("Successfully created group: " + ret.getName());

    		return ret;
    	}
    	catch(InvalidSessionException e){
            // Handle exception
            System.err.println(e.getMessage());
    	}
    	catch(InsufficientPrivilegesException e){
            // Handle exception
            System.err.println(e.getMessage());
    	}
    	catch(InvalidGroupException e){
            // Handle exception
            System.err.println(e.getMessage());
    	}
    	catch(RemoteException e){
            // Handle exception
            System.err.println(e.getMessage());
    	}
    	catch(InvalidGroupBeanException e){
            // Handle exception
            System.err.println(e.getMessage());
    	}
    	catch(InvalidUserException e){
    		// Handle exception
            System.err.println(e.getMessage());
    	}

    	return null;
    }

    //gets the members of a given group
    public void getMembersOfGroup(String groupName){
    	try{
    		RemoteGroup group = groupService.get(new GroupBean(groupName));

    		System.out.println("Users:");

    		List users = group.getUsers();
    		for(Iterator iter = users.iterator(); iter.hasNext(); ){
    			System.out.println(iter.next());
    		}
    	}
    	catch(InvalidSessionException e){
            // Handle exception
            System.err.println(e.getMessage());
    	}
    	catch(InsufficientPrivilegesException e){
            // Handle exception
            System.err.println(e.getMessage());
    	}
    	catch(InvalidGroupException e){
            // Handle exception
            System.err.println(e.getMessage());
    	}
    	catch(RemoteException e){
            // Handle exception
            System.err.println(e.getMessage());
    	}
    	catch(InvalidGroupBeanException e){
            // Handle exception
            System.err.println(e.getMessage());
    	}
    }

    //adds the given user to the given group
    public void updateMembers(String groupName, String userId){
    	try{
    		GroupBean groupBean = new GroupBean(groupName);
    		groupBean.addUser(userId);
    		groupService.update(groupBean);
    	}
    	catch(InvalidSessionException e){
            // Handle exception
            System.err.println(e.getMessage());
    	}
    	catch(InsufficientPrivilegesException e){
            // Handle exception
            System.err.println(e.getMessage());
    	}
    	catch(InvalidGroupException e){
            // Handle exception
            System.err.println(e.getMessage());
    	}
    	catch(RemoteException e){
            // Handle exception
            System.err.println(e.getMessage());
    	}
    	catch(InvalidGroupBeanException e){
            // Handle exception
            System.err.println(e.getMessage());
    	}
    	catch(ObjectModifiedException e){
    		// Handle exception
            System.err.println(e.getMessage());
    	}
    	catch(InvalidUserException e){
    		// Handle exception
            System.err.println(e.getMessage());
    	}
    }

    public static void main(String[] argv)
    {
        URL serverURL = null;
        String serverURLString = null;
        String userid = null;
        String password = null;


        if (argv.length < 1) {
            showUsage();
            System.exit(1);
        }

        for (int i = 0; i < argv.length; i++) {
            if (argv[i].equalsIgnoreCase("-s")) {
                i++;
                if (i < argv.length) serverURLString = argv[i];
            } else if (argv[i].equalsIgnoreCase("-u")) {
                i++;
                if (i < argv.length) userid = argv[i];
            } else if (argv[i].equalsIgnoreCase("-p")) {
                i++;
                if (i < argv.length) password = argv[i];
            }
        }


        if (serverURLString == null) {
            serverURLString = String.valueOf(readInput("Server URL: "));
        }
        try {
            serverURL = new URL(serverURLString);
        } catch (MalformedURLException me) {
            System.err.println("The server URL is not valid.");
            showUsage();
            System.exit(1);
        }
        if (userid == null || userid.length() == 0) {
            userid = String.valueOf(readInput("Username: "));
        }
        if (password == null || String.valueOf(password).length() == 0) {
            password = readInput("Password: ");
        }

        SASDrugDevRemoteAPIAdminSample sddRemoteAPIClient = new SASDrugDevRemoteAPIAdminSample(serverURL, userid, password);
        if ( sddRemoteAPIClient.isConnected() )
        {
            System.out.println("Connected to the server..\n");
        }
        else
        {
            System.out.println("Error connecting to server");
            System.exit(1);
        }


        boolean exitSample = false;

        try
        {
            while ( ! exitSample)
            {
                try {
                    System.out.println("\n\nPlease select a command to execute: ");
                    showCommands();
                    System.out.println("\n");
                    int commandName = Integer.valueOf(readInput("Command : ")).intValue();

                    switch (commandName) {
                    case COMMAND_GET_USER:
                    {
                    	String userId = String.valueOf(readInput("SAS Drug Dev User Id : "));
                        sddRemoteAPIClient.getUser(userId);
                        break;
                    }
                    case COMMAND_CREATE_USER:
                    {
                    	String userId = String.valueOf(readInput("SAS Drug Dev User Id : "));
                    	String pwd = String.valueOf(readInput("Password : "));
                    	String firstName = String.valueOf(readInput("First name : "));
                        String lastName = String.valueOf(readInput("Last name : "));
                        String email = String.valueOf(readInput("Email Address : "));
                        sddRemoteAPIClient.createUser(userId, pwd, firstName, lastName, email);
                        break;
                    }
                    case COMMAND_USER_GET_POLICIES:
                    {
                    	String userId = String.valueOf(readInput("SAS Drug Dev User Id : "));
                        sddRemoteAPIClient.getUserPolicies(userId);
                        break;
                    }
                    case COMMAND_USER_UPDATE_POLICIES:
                    {
                      	String userId = String.valueOf(readInput("SAS Drug Dev User Id (this will add the DataExplorer and JobEditor policies): "));
                        sddRemoteAPIClient.updateUserPolicies(userId);
                        break;
                    }
                    case COMMAND_USER_GET_PROPS:
                    {
                    	String userId = String.valueOf(readInput("SAS Drug Dev User Id : "));
                        sddRemoteAPIClient.getUserProperties(userId);
                        break;
                    }
                    case COMMAND_USER_SET_PROPS:
                    {
                    	String userId = String.valueOf(readInput("SAS Drug Dev User Id : "));
                        String propertyName = String.valueOf(readInput("Name of user property to be changed : "));
                        String propertyValue = String.valueOf(readInput("New value : "));
                        sddRemoteAPIClient.setUserProperties(userId, propertyName, propertyValue);

                        break;
                    }
                    case COMMAND_GET_GROUP:
                    {
                        String group = String.valueOf(readInput("Group Name : "));
                        sddRemoteAPIClient.getGroupFromSDD(group);
                        break;
                    }
                    case COMMAND_CREATE_GROUP:
                    {
                        String group = String.valueOf(readInput("Group Name : "));
                        String desc = String.valueOf(readInput("Description : "));
                        sddRemoteAPIClient.createGroup(group, desc);
                        break;
                    }
                    case COMMAND_GET_MEMBERS_OF_GROUP:
                    {
                        String group = String.valueOf(readInput("Group Name : "));
                        sddRemoteAPIClient.getMembersOfGroup(group);
                        break;
                    }
                    case COMMAND_UPDATE_MEMBERS_OF_GROUP:
                    {
                    	String group = String.valueOf(readInput("Group Name : "));
                    	String userId = String.valueOf(readInput("Add user Id : "));
                        sddRemoteAPIClient.updateMembers(group, userId);
                        break;
                    }
                    case COMMAND_CHANGE_CURRENT_PASSWORD:
                    {
                    	String currentPassword = String.valueOf(readInput("Current Password : "));
                    	String newPassword = String.valueOf(readInput("New Password : "));
                        sddRemoteAPIClient.changeCurrentUserPassword(currentPassword.toCharArray(), newPassword.toCharArray());
                        break;
                    }
                    case COMMAND_GET_PASSWORD_EXPIRATION_DATE:
                    {
                        sddRemoteAPIClient.checkPasswordExpirationDate();
                        break;
                    }
                    case COMMAND_TEST_SESSION:
                    {
                        if (sddRemoteAPIClient.isConnected()) {
                            System.out.println("Connected to the server!");
                        } else {
                            System.out.println();
                            System.out.println("**Connection to " + serverURL + " is not valid.**");
                            System.out.println();
                        }
                        break;
                    }
                    case COMMAND_EXIT:
                    {
                        exitSample = true;
                        break;
                    }
                    default:
                    {
                        System.err.println("\n An Invalid Command was entered! \n");
                        System.out.println("Available commands are:\n");
                        showCommands();
                        break;
                    }
                    }
                } catch (Exception e) {
                    System.out.println("A failure occured while processing this command, error message is :");
                    System.out.println(e.getMessage());
                    e.printStackTrace();

                }
            }

        }
        finally
        {
            sddRemoteAPIClient.closeSASDrugDevSession();
        }

        System.exit(0);
    }

    private static void showUsage()
    {
        System.out.println("Usage: SASDrugDevRemoteAPIAdminSamples -s <server URL> [-u <username>] [-p <password>]");
    }

    private static void showCommands()
    {
        System.out.println("1 : Get a user from SDD");
        System.out.println("2 : Create a new user");
        System.out.println("3 : Get a user's policies");
        System.out.println("4 : Update a users's policies");
        System.out.println("5 : Get a user's properties");
        System.out.println("6 : Set a user's properties");
        System.out.println("7 : Get a group from SDD");
        System.out.println("8 : Create a new group");
        System.out.println("9 : Get a group's members");
        System.out.println("10 : Update a group's members");
        System.out.println("11 : Change the current user's password");
        System.out.println("12 : Get the current user's password expiration date");
        System.out.println("13 : Check if connection to server is valid");
        System.out.println("14 : Exit the Remote API Sample Program");
    }

    private static String readInput(String prompt)
    {
        char[] ret = new char[] {'\0'};
        try {
            BufferedReader in = new BufferedReader(new InputStreamReader(System.in));
            System.out.print(prompt);
            ret = (in.readLine()).toCharArray();
        } catch (IOException e) {
            System.err.println("Error reading input.");
        }
        return new String(ret);
    }

	// Change password of the current user
	public void changeCurrentUserPassword(char[] currentPassword,
			char[] newPassword) {

		try {
			// Using the UserService, change password of userid that was used to establish a session with the SAS Drug Dev instance
			userService.changePassword(currentPassword, newPassword);
			System.out
					.println("Password for the current user has been changed to the new password");

		} catch (InvalidPasswordException e) {
			// Handle exception
			System.err.println(e.getMessage());
		} catch (RemoteException e) {
			// Handle exception
			System.err.println(e.getMessage());
		} catch (InvalidSessionException e) {
			// Handle exception
			System.err.println(e.getMessage());
		}
	}

//	 Retrieve the password expiration date of userid that was used to establish a session with the SAS Drug Dev instance
	public void checkPasswordExpirationDate() {
		try {
			// Use the AdminService to get the password expiration date for current user
			Date expirationDate = userService.getPasswordExpirationDate();

			SimpleDateFormat dateFormat = new SimpleDateFormat();
			System.out
					.println("\n Password of the current user is set to expire on "
							+ new SimpleDateFormat().format(expirationDate));

		} catch (RemoteException e) {
			// Handle exception
			System.err.println(e.getMessage());
		} catch (InvalidSessionException e) {
			// Handle exception
			System.err.println(e.getMessage());
		}

	}



}
